@@ -12,6 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from datetime import date
 from pathlib import Path
 from typing import Any, Dict, List, NamedTuple, Optional, Set, cast
 
@@ -64,7 +65,7 @@ class Generator(AbstractODSGenerator):
     __tax_sheet_year_2_row: Dict[_AssetAndYear, int] = {}
 
     __in_header_names_row_1: List[str] = []
-    __in_header_names_row: List[str] = []
+    __in_header_names_row_2: List[str] = []
     __out_header_names_row_1: List[str] = []
     __out_header_names_row_2: List[str] = []
     __intra_header_names_row_1: List[str] = []
@@ -80,7 +81,7 @@ class Generator(AbstractODSGenerator):
 
         currency_code: str = country.currency_iso_code.upper()
 
-        self.__in_header_names_row_1 = [
+        self.__in_header_names_row_1: List[str] = [
             "",
             "",
             "",
@@ -98,7 +99,7 @@ class Generator(AbstractODSGenerator):
             "",
         ]
 
-        self.__in_header_names_row = [
+        self.__in_header_names_row_2: List[str] = [
             "Sent/Sold",
             "Timestamp",
             "Asset",
@@ -116,7 +117,7 @@ class Generator(AbstractODSGenerator):
             "Notes",
         ]
 
-        self.__out_header_names_row_1 = [
+        self.__out_header_names_row_1: List[str] = [
             "",
             "",
             "",
@@ -133,7 +134,7 @@ class Generator(AbstractODSGenerator):
             "",
         ]
 
-        self.__out_header_names_row_2 = [
+        self.__out_header_names_row_2: List[str] = [
             "Timestamp",
             "Asset",
             "Exchange",
@@ -150,7 +151,7 @@ class Generator(AbstractODSGenerator):
             "Notes",
         ]
 
-        self.__intra_header_names_row_1 = [
+        self.__intra_header_names_row_1: List[str] = [
             "",
             "",
             "From",
@@ -167,7 +168,7 @@ class Generator(AbstractODSGenerator):
             "",
         ]
 
-        self.__intra_header_names_row_2 = [
+        self.__intra_header_names_row_2: List[str] = [
             "Timestamp",
             "Asset",
             "Exchange",
@@ -184,7 +185,7 @@ class Generator(AbstractODSGenerator):
             "Notes",
         ]
 
-        self.__balance_header_names_row_1 = [
+        self.__balance_header_names_row_1: List[str] = [
             "",
             "",
             "",
@@ -194,7 +195,7 @@ class Generator(AbstractODSGenerator):
             "Final",
         ]
 
-        self.__balance_header_names_row_2 = [
+        self.__balance_header_names_row_2: List[str] = [
             "Exchange",
             "Holder",
             "Asset",
@@ -204,7 +205,7 @@ class Generator(AbstractODSGenerator):
             "Balance",
         ]
 
-        self.__gain_loss_summary_header_names_row_1 = [
+        self.__gain_loss_summary_header_names_row_1: List[str] = [
             "",
             "",
             "Capital",
@@ -215,7 +216,7 @@ class Generator(AbstractODSGenerator):
             f"{currency_code} Total",
         ]
 
-        self.__gain_loss_summary_header_names_row_2 = [
+        self.__gain_loss_summary_header_names_row_2: List[str] = [
             "Year",
             "Asset",
             "Gains",
@@ -226,7 +227,7 @@ class Generator(AbstractODSGenerator):
             "Cost Basis",
         ]
 
-        self.__gain_loss_detail_header_names_row_1 = [
+        self.__gain_loss_detail_header_names_row_1: List[str] = [
             "Crypto",
             "",
             "Crypto Amt",
@@ -247,7 +248,7 @@ class Generator(AbstractODSGenerator):
             "Acquired Lot Fraction",
         ]
 
-        self.__gain_loss_detail_header_names_row_2 = [
+        self.__gain_loss_detail_header_names_row_2: List[str] = [
             "Amount",
             "Asset",
             "Running Sum",
@@ -275,6 +276,8 @@ class Generator(AbstractODSGenerator):
         asset_to_computed_data: Dict[str, ComputedData],
         output_dir_path: str,
         output_file_prefix: str,
+        from_date: date,
+        to_date: date,
     ) -> None:
 
         if not isinstance(asset_to_computed_data, Dict):
@@ -290,6 +293,8 @@ class Generator(AbstractODSGenerator):
             output_file_prefix=output_file_prefix,
             output_file_name=self.OUTPUT_FILE,
             template_sheets_to_keep=self.TEMPLATE_SHEETS_TO_KEEP,
+            from_date=from_date,
+            to_date=to_date,
         )
 
         asset: str
@@ -377,7 +382,7 @@ class Generator(AbstractODSGenerator):
         return _BorderStyle(year, border_suffix)
 
     def __generate_in_table(self, sheet: Any, computed_data: ComputedData, row_index: int) -> int:
-        row_index = self._fill_header("In-Flow Detail", self.__in_header_names_row_1, self.__in_header_names_row, sheet, row_index, 0)
+        row_index = self._fill_header("In-Flow Detail", self.__in_header_names_row_1, self.__in_header_names_row_2, sheet, row_index, 0)
 
         in_transaction_set: TransactionSet = computed_data.in_transaction_set
         entry: AbstractEntry
@@ -457,7 +462,7 @@ class Generator(AbstractODSGenerator):
             self._fill_cell(sheet, row_index, 9, computed_data.get_crypto_out_running_sum(transaction), data_style="crypto", visual_style=visual_style)
             self._fill_cell(sheet, row_index, 10, computed_data.get_crypto_out_fee_running_sum(transaction), data_style="crypto", visual_style=visual_style)
             self._fill_cell(sheet, row_index, 11, transaction.crypto_out_no_fee * transaction.spot_price, visual_style=highlighted_style, data_style="fiat")
-            self._fill_cell(sheet, row_index, 12, transaction.crypto_fee * transaction.spot_price, visual_style=highlighted_style, data_style="fiat")
+            self._fill_cell(sheet, row_index, 12, transaction.crypto_fee * transaction.spot_price, visual_style=visual_style, data_style="fiat")
             self._fill_cell(sheet, row_index, 13, "YES" if transaction.is_taxable() else "NO", data_style="fiat", visual_style=visual_style)
             self._fill_cell(sheet, row_index, 14, transaction.notes, visual_style="transparent")
 
