@@ -16,7 +16,7 @@ import cProfile
 from datetime import date
 import os
 import sys
-from argparse import ArgumentParser, Namespace
+from argparse import ArgumentParser, Namespace, RawTextHelpFormatter
 from importlib import import_module
 from pathlib import Path
 from pkgutil import iter_modules
@@ -103,6 +103,8 @@ def _rp2_main_internal(country: AbstractCountry) -> None:
             country=country,
             accounting_method=accounting_method,
             asset_to_computed_data=asset_to_computed_data,
+            from_date=configuration.from_date,
+            to_date=configuration.to_date,
         )
         # Run country-specific report generators
         _find_and_run_report_generators(
@@ -111,6 +113,8 @@ def _rp2_main_internal(country: AbstractCountry) -> None:
             country=country,
             accounting_method=accounting_method,
             asset_to_computed_data=asset_to_computed_data,
+            from_date=configuration.from_date,
+            to_date=configuration.to_date,
         )
     except Exception:  # pylint: disable=broad-except
         LOGGER.exception("Fatal exception occurred:")
@@ -121,7 +125,13 @@ def _rp2_main_internal(country: AbstractCountry) -> None:
 
 
 def _find_and_run_report_generators(
-    package_path: str, args: Namespace, country: AbstractCountry, accounting_method: AbstractAccountingMethod, asset_to_computed_data: Dict[str, ComputedData]
+    package_path: str,
+    args: Namespace,
+    country: AbstractCountry,
+    accounting_method: AbstractAccountingMethod,
+    asset_to_computed_data: Dict[str, ComputedData],
+    from_date: date,
+    to_date: date,
 ) -> None:
     # Load report generator plugins and call their generate() method
     package: ModuleType = import_module(package_path)
@@ -147,6 +157,8 @@ def _find_and_run_report_generators(
                 asset_to_computed_data=asset_to_computed_data,
                 output_dir_path=args.output_dir,
                 output_file_prefix=args.prefix,
+                from_date=from_date,
+                to_date=to_date,
             )
         package_found = True
 
@@ -179,10 +191,13 @@ def _find_accounting_methods() -> List[str]:
 def _setup_argument_parser(accounting_methods: List[str]) -> ArgumentParser:
     parser: ArgumentParser = ArgumentParser(
         description=(
-            "Generate yearly capital gain/loss report and account balances for crypto holdings. "
-            "If this software is useful to you, consider leaving a star on its Github page: "
-            "https://github.com/eprbell/rp2"
-        )
+            "Generate capital gain/loss report and balances for crypto holdings. Links:\n"
+            "- documentation: https://github.com/eprbell/rp2/blob/main/README.md\n"
+            "- FAQ: https://github.com/eprbell/rp2/blob/main/docs/user_faq.md\n"
+            "- donations: https://eprbell.github.io/eprbell/donations.html\n"
+            "- leave a star on Github: https://github.com/eprbell/rp2"
+        ),
+        formatter_class=RawTextHelpFormatter,
     )
 
     parser.add_argument(
